/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for Y-ASK-V850E2-FG4 starter kit
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"
#include "io70f3580.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Assign LEDs to Ports ****************************************/

/****** Target specific configuration *******************************/

#define LEDPORT_PMC  (PMC1)
#define LEDPORT_PIPC (PIPC1)
#define LEDPORT_PM   (PM1)
#define LEDPORT_SET  (PSR1)
#define LEDPORT_CLR  (PSR1)
#define LEDPORT_NOT  (PNOT1)
#define LEDPORT_OUT  (P1)
#define LEDPORT_IN   (PPR1)

#define LED0_BIT      (3)  /* Blue LED1, active high */
#define LED1_BIT      (4)  /* Blue LED2, active high */
#define LED2_BIT      (5)  /* Blue LED3, active high */
#define LED3_BIT      (6)  /* Blue LED4, active high */

#define LEDMASK_ALL   ((1u << LED0_BIT) | (1u << LED1_BIT) | (1u << LED2_BIT) | (1u << LED3_BIT))

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  LEDPORT_PMC  &= ~(LEDMASK_ALL);  // Switch all LED ports to port mode
  LEDPORT_PIPC &= ~(LEDMASK_ALL);  // Data direction controlled by port mode
  LEDPORT_PM   &= ~(LEDMASK_ALL);  // Switch LED ports to output mode
  LEDPORT_OUT  &= ~(LEDMASK_ALL);  // Initially switch off all LEDs
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {   /* Switch on LED  */
  LEDPORT_OUT = LEDPORT_IN | (LEDMASK_ALL & (1u << (LED0_BIT + Index)));
}

void BSP_ClrLED(int Index) {   /* Switch off LED */
  LEDPORT_OUT = LEDPORT_IN & ~(LEDMASK_ALL & (1u << (LED0_BIT + Index)));
}

void BSP_ToggleLED(int Index) {
  LEDPORT_NOT = LEDMASK_ALL & (1u << (LED0_BIT + Index));
}

/****** End Of File *************************************************/


