/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File   : RTOSInit_D70F37xxGJ.c
Purpose: Initializes and handles the hardware for embOS as far
         as required by embOS for NEC D70F37xxGJ CPU.
         Feel free to modify this file acc. to your
         target system.
-------- END-OF-HEADER -----------------------------------------------
*/

#include "RTOS.H"

#include <io70f3746.h>          /* SFR file delivered from IAR      */

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU: used to initilize embOS timer
  and baudrate for optional UART for embOSView

  Set UART and Baudrate for embOSView
  If you do not want (or can not due to hardware limitations) to
  dedicate a UART to embOSView, please define OS_UART to -1

*/
#ifndef   OS_FSYS
  #define OS_FSYS 16000000  /* default is 16 MHz, depends on crystal and PLL setting */
#endif
#ifndef   OS_UART
  #define OS_UART (-1)
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER OS_FSYS
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART  OS_FSYS
#endif

#ifndef   INIT_PLL
  #define INIT_PLL 1
#endif

/****** End of configuration settings *******************************/

#define OS_UART_USED (OS_UART == 3)

/****** SFRs used in RTOSInit are define here ***********************/

/****** Timer used as embOS timer ***********************************/

#define TP0CTL0    *((volatile unsigned char*) 0xFFFFF590)
#define TP0CTL1    *((volatile unsigned char*) 0xFFFFF591)
#define TP0IOC0    *((volatile unsigned char*) 0xFFFFF592)
#define TP0IOC1    *((volatile unsigned char*) 0xFFFFF593)
#define TP0IOC2    *((volatile unsigned char*) 0xFFFFF594)
#define TP0OPT0    *((volatile unsigned char*) 0xFFFFF595)
#define TP0CCR0    *((volatile unsigned short*)0xFFFFF596)
#define TP0CCR1    *((volatile unsigned short*)0xFFFFF598)
#define TP0CNT     *((volatile unsigned short*)0xFFFFF59A)
#define TP0CCIC0   *((volatile unsigned char*) 0xFFFFF12E)
#define WDTM2      *((volatile unsigned char*) 0xFFFFF6D0)
#define VSWC       *((volatile unsigned char*) 0xFFFFF06E)


/****** UART for communication with embOSView ***********************/
#if OS_UART_USED

#define UA3RIC    *((volatile unsigned char*) 0xFFFFF1A6)
#define UA3TIC    *((volatile unsigned char*) 0xFFFFF1A8)
#define CB4CTL0   *((volatile unsigned char*) 0xFFFFFD40)
#define UA3CTL0   *((volatile unsigned char*) 0xFFFFFA30)
#define UA3CTL1   *((volatile unsigned char*) 0xFFFFFA31)
#define UA3CTL2   *((volatile unsigned char*) 0xFFFFFA32)
#define UA3OPT0   *((volatile unsigned char*) 0xFFFFFA33)
#define UA3STR    *((volatile unsigned char*) 0xFFFFFA34)
#define UA3RX     *((volatile unsigned char*) 0xFFFFFA36)
#define UA3TX     *((volatile unsigned char*) 0xFFFFFA37)
#define PMC8      *((volatile unsigned short*)0xFFFFF450)

/*  baud = OS_FSYS/[2^(BAUD_M) * BAUD_K] */
#if (OS_PCLK_UART == 32000000)
  #define OS_UART_DIVIDER 0xD0
  #if   (OS_BAUDRATE == 38400)
    #define OS_UART_PRESCALE 0x01
  #elif (OS_BAUDRATE == 19200)
    #define OS_UART_PRESCALE 0x02
  #elif (OS_BAUDRATE == 9600)
    #define OS_UART_PRESCALE 0x03
  #elif (OS_BAUDRATE == 4800)
    #define OS_UART_PRESCALE 0x04
  #elif (OS_BAUDRATE == 2400)
    #define OS_UART_PRESCALE 0x05
  #else
    #define OS_BAUDRATE_ERROR 1
  #endif
#elif (OS_PCLK_UART == 20000000)
  #define OS_UART_DIVIDER 0x82
  #if   (OS_BAUDRATE == 38400)
    #define OS_UART_PRESCALE 0x01
  #elif (OS_BAUDRATE == 19200)
    #define OS_UART_PRESCALE 0x02
  #elif (OS_BAUDRATE == 9600)
    #define OS_UART_PRESCALE 0x03
  #elif (OS_BAUDRATE == 4800)
    #define OS_UART_PRESCALE 0x04
  #elif (OS_BAUDRATE == 2400)
    #define OS_UART_PRESCALE 0x05
  #else
    #define OS_BAUDRATE_ERROR 1
  #endif
#elif (OS_PCLK_UART == 16000000)
  #define OS_UART_DIVIDER 0x68
  #if   (OS_BAUDRATE == 38400)
    #define OS_UART_PRESCALE 0x01
  #elif (OS_BAUDRATE == 19200)
    #define OS_UART_PRESCALE 0x02
  #elif (OS_BAUDRATE == 9600)
    #define OS_UART_PRESCALE 0x03
  #elif (OS_BAUDRATE == 4800)
    #define OS_UART_PRESCALE 0x04
  #elif (OS_BAUDRATE == 2400)
    #define OS_UART_PRESCALE 0x05
  #else
    #define OS_BAUDRATE_ERROR 1
  #endif
#elif (OS_PCLK_UART == 10000000)
  #define OS_UART_DIVIDER 0x82
  #if   (OS_BAUDRATE == 38400)
    #define OS_UART_PRESCALE 0x00
  #elif (OS_BAUDRATE == 19200)
    #define OS_UART_PRESCALE 0x01
  #elif (OS_BAUDRATE == 9600)
    #define OS_UART_PRESCALE 0x02
  #elif (OS_BAUDRATE == 4800)
    #define OS_UART_PRESCALE 0x03
  #elif (OS_BAUDRATE == 2400)
    #define OS_UART_PRESCALE 0x04
  #else
    #define OS_BAUDRATE_ERROR 1
  #endif
#else
  #define OS_BAUDRATE_ERROR 1
#endif
#ifdef OS_BAUDRATE_ERROR
  #error "Baudrate settings not supported, please define OS_UART_PRESCALE and OS_UART_DIVIDER"
#endif

#endif /* OS_UART_USED */


/*********************************************************************
*
*       Locals
*
**********************************************************************
*/

#if OS_UART_USED
/*********************************************************************
*
*       _ISR_Rx_Handler,  receive interrupt handler
*/
static void _ISR_Rx_Handler(void) {
  volatile unsigned char Dummy;

  if ((UA3STR & 0x07) == 0) {   /* No error                         */
    OS_OnRx(UA3RX);             /* Process data byte                */
  } else {
    UA3STR &= ~0x07;            /* Reset error                      */
    Dummy   = UA3RX;            /* discard Byte                     */
  }
}
#endif

/*********************************************************************
*
*       Globals
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Initialize the hardware required for the OS to run. This will work
*  on any target hardware, but may have to be tailored a little
*  (regarding the clock frequency). Of course the same holds true
*  if for some reason you choose to use an other timer.
*/
#define TIMER_RELOAD (OS_PCLK_TIMER/1000 - 1)

void OS_InitHW(void) {
  VSWC     = 0x01;           /* mainclock (2MHz, 16.6MHz) 1wait             */
  WDTM2    = 0x00;           /* disable watchdog timer 2                    */
  TP0CTL0  = 0x00;           /* stop timer TMP0 for initializatio           */
  TP0CTL1  = 0x00;           /* interval timer mode                         */
  TP0IOC0  = 0x00;           /* timer output (not used)                     */
  TP0IOC1  = 0x00;           /* timer capture trigger input (not used)      */
  TP0IOC2  = 0x00;           /* timer event count input (not used)          */
  TP0OPT0  = 0x10;           /* cc register selection, ovf (not used)       */
  TP0CCR0  = TIMER_RELOAD;   /* compare register 0 set to 1ms resolution    */
  TP0CCIC0 = 0x07;           /* enable timerp0 interrupt0, lowest priority  */
  TP0CTL0  = 0x80;           /* enable timer, prescale 1                    */
  OS_COM_Init();
}

/*********************************************************************
*
*       OS_Idle() Idle Loop
*
*  Please note:
*  This is basically the "core" of the idle loop.
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {     /* Idle loop: No task is ready to exec     */
  while (1) {
  };
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*  This routine is required for task-info via embOS Viewer or
*  high resolution time measurement functions.
*  It returns the system time in clock cycles of hardware timer.
*/
#define HW_TIMER TP0CNT
#define HW_TIMER_INTERRUPT_PENDING() (TP0CCIC0 & 0x80)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = HW_TIMER;
  OS_U32 time        = OS_Time;

  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = HW_TIMER;
    if (t_cnt == TIMER_RELOAD) {
      t_cnt = 0;
    }
    time++;
  }
  return (OS_PCLK_TIMER / 1000) * time + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*  Purpose:
*     Convert timer cycles into micro seconds.
*     Used for Profiling or high resolution time measurement.
*     If you use a clock frequency which is not a multiple of 1 MHz,
*     you may have to modify this routine in order to get proper
*     diagonstics.
*  Please note:
*     This routine is required for profiling and high resolution time
*     measurement.
*     It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick() interrupt handler
*/
#pragma vector=INTTP0CC0_vector
__interrupt void OS_ISR_Tick(void); /* Avoid "no prototype" error */
__interrupt void OS_ISR_Tick(void) {
  OS_EnterNestableInterrupt();
  OS_HandleTick();
  OS_LeaveNestableInterrupt();
}

#if (OS_UART_USED)
/*********************************************************************
*
*       Communication for embOSView
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  volatile unsigned char Dummy;

  /*  Initially disable interrupts */
  OS_DI();                                       /* Disable interrupts                        */
  UA3RIC   = 0x47;                               /* Disable Rx interrupt, set lowest priority */
  UA3TIC   = 0x47;                               /* Disable Tx interrupt, set lowest priority */
  /*  Initially disable UART A0 */
  CB4CTL0  = 0x01;                               /* Disable CSI 4 which shares same port */
  UA3CTL0 &= ~(1 << 6);                          /* Initially disable Tx */
  UA3CTL0 &= ~(1 << 5);                          /* Disable Rx */
  UA3CTL0 &= ~(1 << 7);                          /* Disable Uart */
  PMC8 |= 0x03;
  /*  Setup UART control registers */
  UA3CTL0  = (1 << 4)                            /* LSB first */
           | (0 << 3)                            /* No parity during Tx */
           | (0 << 2)                            /* No parity during Rx */
           | (1 << 1)                            /* 8 bits per character */
           | (0 << 0);                           /* 1 Stop bit */
  UA3OPT0  = 0x14;                               /* Default value */
  UA3STR   = 0x00;                               /* Reset Transmit Flag and all errors */
  Dummy    = UA3RX;                              /* Dummy read to clear Rx buffer */
  /*  Initialize Baudrate generator */
  UA3CTL1  = OS_UART_PRESCALE;
  UA3CTL2  = OS_UART_DIVIDER;
  /*  Enable UART, Rx, Tx */
  UA3CTL0 |=  (1 << 7);                          /* Enable Uart     */
  /*  Wait two base clock cycles                                    */
  UA3CTL0 |=  (1 << 6);                          /* Enable Tx       */
  UA3CTL0 |=  (1 << 5);                          /* Enable Rx       */
  UA3RIC  &= ~0x40;                              /* Enable RX ISR   */
  UA3TIC  &= ~0x40;                              /* Enable TX ISR   */
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_COM_Send1
*
Purpose: Transmitter Routine used by embOS for communication with
         embOSView.

Caution: NEVER call this routine from your application.

This routine depends on the serial interface used for communication
with embOSView. Therefore it is public here, so it can be modified
if you have to use an other UART for embOSView.
*/
void OS_COM_Send1(unsigned char c) {
  UA3TX = c;
}

/*********************************************************************
*
*       OS_ISR_tx(), transmit interrupt handler
*/
#pragma vector=INTUA3T_vector
__interrupt void OS_ISR_tx(void); /* Avoid "no prototype" error */

__interrupt void OS_ISR_tx(void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       OS_ISR_Rx(), receive interrupt
*/
#pragma vector=INTUA3R_vector
__interrupt void OS_ISR_rx(void); /* Avoid "no prototype" error */

__interrupt void OS_ISR_rx(void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  _ISR_Rx_Handler();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

#else  /* No supported UART selected; use dummy routines            */

/*********************************************************************
*
*       Dummy communication routines
*
**********************************************************************
*/
void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid warning       */
  OS_COM_ClearTxActive();  /* Release transmitter */
}

#endif // OS_UART_USED

/********************************************************************/


/*********************************************************************
*
*       __low_level_init()
*
*       Initialize clock generation and pll
*
*       Has to be modified, if another CPU clock frequency should be
*       used. This function is called during startup and
*       has to return 1 to perform segment initialization
*/
unsigned char __low_level_init(void) {
  #if INIT_PLL
    OS_InitPLL();
  #endif
  //
  // Other hardware initialization may be added here.
  //
  return 1;
}

/****** End of file *************************************************/
